#include <svn/client.hh>
#include <svn/pool.hh>
#include <svn/exception.hh>
#include <svn/wc.hh>
#include <svn_opt.h>

using namespace std;

namespace svn {
namespace client {

const RevisionSpecifier RevisionSpecifier::HEAD(RevisionSpecifier::rHead);

RevisionSpecifier::RevisionSpecifier(Type t) {
    me.kind = static_cast<svn_opt_revision_kind>(t);
    me.value.number = SVN_INVALID_REVNUM;
}

svn_error_t *AuthBaton::prompt_callback(char **info, const char *prompt,
        svn_boolean_t hide, void *baton, apr_pool_t *pool) {
    AuthBaton *myThis = reinterpret_cast<AuthBaton*>(baton);
    try {
        Pool poolParam(pool);
        std::string infoStr = myThis->prompt(string(prompt),
                static_cast<bool>(hide), poolParam);
        *info = apr_pstrdup(pool, infoStr.c_str());
    } catch (svn::Exception &e) {
        abort();
    }
    return SVN_NO_ERROR;
}

AuthBaton::AuthBaton() {
    memset(&me, 0, sizeof(svn_client_auth_baton_t));
    me.prompt_callback = &prompt_callback;
    me.prompt_baton = this;
}

static void notify_func(void *baton, const char *path,
        svn_wc_notify_action_t action, svn_node_kind_t kind,
        const char *mime_type, svn_wc_notify_state_t content_state,
        svn_wc_notify_state_t prop_state, svn_revnum_t revision) {
    boost::function1<void,const wc::Notification&> *notifyFunc =
        reinterpret_cast<boost::function1<void,const wc::Notification&>*>
        (baton);
    wc::Notification n;
    n.path = path;
    n.action = static_cast<wc::NotifyAction>(action);
    n.kind = static_cast<NodeKind>(kind);
    if (mime_type != NULL)
        n.mimeType = mime_type;
    n.contentState = static_cast<wc::NotifyState>(content_state);
    n.propertiesState = static_cast<wc::NotifyState>(prop_state);
    n.revision = revision;
    (*notifyFunc)(n);
}

void checkout(boost::function1<void,const wc::Notification&> notifyFunc,
        AuthBaton authBaton, const std::string &url, const std::string &path,
        const RevisionSpecifier &revision, bool recurse, Pool &pool) {
    Exception::handle(svn_client_checkout(&notify_func, &notifyFunc,
            &authBaton.me, url.c_str(), path.c_str(), &revision.me, recurse,
            pool));
}

void update(AuthBaton authBaton, const std::string &path,
        const RevisionSpecifier &revision, bool recurse,
        boost::function1<void,const wc::Notification&> notifyFunc, Pool &pool) {
    Exception::handle(svn_client_update(&authBaton.me, path.c_str(),
            &revision.me, recurse, &notify_func, &notifyFunc, pool));
}

}
}
